"""
| YATE engine
|-------------
| MIT License
|
| Copyright (c) 2022 Mauro Baladés
|
| Permission is hereby granted, free of charge, to any person obtaining a copy
| of this software and associated documentation files (the "Software"), to deal
| in the Software without restriction, including without limitation the rights
| to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
| copies of the Software, and to permit persons to whom the Software is
| furnished to do so, subject to the following conditions:
|
| The above copyright notice and this permission notice shall be included in all
| copies or substantial portions of the Software.
|
| THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
| IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
| FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
| AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
| LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
| OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
| SOFTWARE.
"""

import operator
import re

# We will encapsulate each fragment of text in a Fragment object.
# This object will determine the fragment type and prepare the
# fragment for consumption by the compile function.
VAR_FRAGMENT = 0
OPEN_BLOCK_FRAGMENT = 1
CLOSE_BLOCK_FRAGMENT = 2
TEXT_FRAGMENT = 3
COMMENT_FRAGMENT = 4

# Variable tokens
VAR_TOKEN_START = "{{"
VAR_TOKEN_END = "}}"

# Code block tokens
BLOCK_TOKEN_START = "{%"
BLOCK_TOKEN_END = "%}"

# Comment block tokens
BLOCK_COMMENT_START = "{#"
BLOCK_COMMENT_END = "#}"

# Token regex
TOK_REGEX = re.compile(
    r"(%s[\S\s]+%s|(%s.*?%s|%s.*?%s))"
    % (
        BLOCK_COMMENT_START,
        BLOCK_COMMENT_END,
        VAR_TOKEN_START,
        VAR_TOKEN_END,
        BLOCK_TOKEN_START,
        BLOCK_TOKEN_END,
    )
)

# White space as a compìled regex object
WHITESPACE = re.compile("\s+")

# Operators
operator_lookup_table = {
    "<": operator.lt,
    ">": operator.gt,
    "==": operator.eq,
    "!=": operator.ne,
    "<=": operator.le,
    ">=": operator.ge,
}
